@@ -4,15 +4,12 @@
  *
  *   Purpose:   C++ class interface for netCDF
  *
- *   $Header: /upc/share/CVS/netcdf-3/cxx/netcdfcpp.h,v 1.5 2001/02/05 20:29:12 russ Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/cxx/netcdfcpp.h,v 1.14 2006/07/31 20:42:48 russ Exp $
  *********************************************************************/
 
 #ifndef NETCDF_HH
 #define NETCDF_HH
 
-extern "C" {
-#include "netcdf.h"             // the C interface
-}
 #include "ncvalues.h"          // arrays that know their element type
 
 typedef const char* NcToken;    // names for netCDF objects
@@ -22,7 +19,6 @@ class NcDim;                    // dimensions
 class NcVar;                    // variables
 class NcAtt;                    // attributes
 
-
 /*
  * ***********************************************************************
  * A netCDF file.
@@ -41,9 +37,18 @@ class NcFile
 	New		// create new file, fail if already exists
       };
 
+    enum FileFormat {
+       Classic,         // netCDF classic format (i.e. version 1 format)
+       Offset64Bits,    // netCDF 64-bit offset format
+       Netcdf4,		// netCDF-4 using HDF5 format
+       Netcdf4Classic,	// netCDF-4 using HDF5 format using only netCDF-3 calls
+       BadFormat
+    };
+
     NcFile( const char * path, FileMode = ReadOnly ,
 	    size_t *chunksizeptr = NULL, // optional tuning parameters
-	    size_t initialsize = 0 );
+	    size_t initialsize = 0,
+	    FileFormat = Classic );
 
     NcBool is_valid( void ) const;         // opened OK in ctr, still valid
 
@@ -62,16 +67,16 @@ class NcFile
     
     // Add new dimensions, variables, global attributes.
     // These put the file in "define" mode, so could be expensive.
-    NcDim* add_dim( NcToken dimname, long dimsize );
-    NcDim* add_dim( NcToken dimname );     // unlimited
+    virtual NcDim* add_dim( NcToken dimname, long dimsize );
+    virtual NcDim* add_dim( NcToken dimname );     // unlimited
 
-    NcVar* add_var( NcToken varname, NcType type,       // scalar
+    virtual NcVar* add_var( NcToken varname, NcType type,       // scalar
                     const NcDim* dim0=0,                // 1-dim
                     const NcDim* dim1=0,                // 2-dim
                     const NcDim* dim2=0,                // 3-dim
                     const NcDim* dim3=0,                // 4-dim
                     const NcDim* dim4=0 );              // 5-dim
-    NcVar* add_var( NcToken varname, NcType type,       // n-dim
+    virtual NcVar* add_var( NcToken varname, NcType type,       // n-dim
                           int ndims, const NcDim** dims );
 
     NcBool add_att( NcToken attname, char );             // scalar attributes
@@ -98,6 +103,7 @@ class NcFile
 
     NcBool set_fill( FillMode = Fill );    // set fill-mode
     FillMode get_fill( void ) const;       // get fill-mode
+    FileFormat get_format( void ) const;   // get format version
 
     NcBool sync( void );                   // synchronize to disk
     NcBool close( void );                  // to close earlier than dtr
@@ -168,6 +174,7 @@ class NcDim
 class NcTypedComponent
 {
   public:
+    virtual ~NcTypedComponent( void ) {}
     virtual NcToken name( void ) const = 0;
     virtual NcType type( void ) const = 0;
     virtual NcBool is_valid( void ) const = 0;
@@ -301,6 +308,7 @@ class NcVar : public NcTypedComponent
     NcBool rename( NcToken newname );
 
     long rec_size ( void );             // number of values per record
+    long rec_size ( NcDim* );           // number of values per dimension slice
 
     // Though following are intended for record variables, they also work
     // for other variables, using first dimension as record dimension.
@@ -308,6 +316,8 @@ class NcVar : public NcTypedComponent
     // Get a record's worth of data
     NcValues *get_rec(void);	        // get current record
     NcValues *get_rec(long rec);        // get specified record
+    NcValues *get_rec(NcDim* d);        // get current dimension slice
+    NcValues *get_rec(NcDim* d, long slice); // get specified dimension slice
 
     // Put a record's worth of data in current record
     NcBool put_rec( const ncbyte* vals );
@@ -318,6 +328,15 @@ class NcVar : public NcTypedComponent
     NcBool put_rec( const float* vals );
     NcBool put_rec( const double* vals );
 
+    // Put a dimension slice worth of data in current dimension slice
+    NcBool put_rec( NcDim* d, const ncbyte* vals );
+    NcBool put_rec( NcDim* d, const char* vals );
+    NcBool put_rec( NcDim* d, const short* vals );
+    NcBool put_rec( NcDim* d, const int* vals );
+    NcBool put_rec( NcDim* d, const long* vals );
+    NcBool put_rec( NcDim* d, const float* vals );
+    NcBool put_rec( NcDim* d, const double* vals );
+
     // Put a record's worth of data in specified record
     NcBool put_rec( const ncbyte* vals, long rec );
     NcBool put_rec( const char* vals, long rec );
@@ -327,6 +346,15 @@ class NcVar : public NcTypedComponent
     NcBool put_rec( const float* vals, long rec );
     NcBool put_rec( const double* vals, long rec );
 
+    // Put a dimension slice worth of data in specified dimension slice
+    NcBool put_rec( NcDim* d, const ncbyte* vals, long slice );
+    NcBool put_rec( NcDim* d, const char* vals, long slice );
+    NcBool put_rec( NcDim* d, const short* vals, long slice );
+    NcBool put_rec( NcDim* d, const int* vals, long slice );
+    NcBool put_rec( NcDim* d, const long* vals, long slice );
+    NcBool put_rec( NcDim* d, const float* vals, long slice );
+    NcBool put_rec( NcDim* d, const double* vals, long slice );
+
     // Get first record index corresponding to specified key value(s)
     long get_index( const ncbyte* vals );
     long get_index( const char* vals );
@@ -336,17 +364,29 @@ class NcVar : public NcTypedComponent
     long get_index( const float* vals );
     long get_index( const double* vals );
 
+    // Get first index of specified dimension corresponding to key values
+    long get_index( NcDim* d, const ncbyte* vals );
+    long get_index( NcDim* d, const char* vals );
+    long get_index( NcDim* d, const short* vals );
+    long get_index( NcDim* d, const int* vals );
+    long get_index( NcDim* d, const long* vals );
+    long get_index( NcDim* d, const float* vals );
+    long get_index( NcDim* d, const double* vals );
+
     // Set current record
     void set_rec ( long rec );
+    // Set current dimension slice
+    void set_rec ( NcDim* d, long slice );
 
     int id( void ) const;               // rarely needed, C interface id
     NcBool sync( void );
     
   private:
+    int dim_to_index(NcDim* rdim);
     int the_id;
     long* the_cur;
     char* the_name;
-    long cur_rec;
+    long* cur_rec;
 
     // private constructors because only an NcFile creates these
     NcVar( void );
@@ -404,9 +444,9 @@ class NcError {
   public:
     enum Behavior {
         silent_nonfatal = 0,
-        verbose_nonfatal = NC_VERBOSE,
-        silent_fatal = NC_FATAL,
-        verbose_fatal = NC_FATAL | NC_VERBOSE      
+        silent_fatal = 1,
+        verbose_nonfatal = 2,
+        verbose_fatal = 3   
       };
 
     // constructor saves previous error state, sets new state
@@ -415,11 +455,15 @@ class NcError {
     // destructor restores previous error state
     virtual ~NcError( void );
 
-    int get_err( void );                 // returns most recent error
+    int get_err( void );                 // returns most recent error number
+    const char* get_errmsg( void ) {return nc_strerror(get_err());}
+    static int set_err( int err );
 
   private:
     int the_old_state;
     int the_old_err;
+    static int ncopts;
+    static int ncerr;
 };
 
 #endif                          /* NETCDF_HH */
